# 基准测试

# libs
- [benchmark](https://github.com/bestiejs/benchmark.js)
- [lodash](https://github.com/lodash/lodash)
- [platform](https://github.com/bestiejs/platform.js)

[benchmark 文档](https://benchmarkjs.com/docs)

## 约定
- 目录名为benchs,与src目录同级
    + benchs目录下`boot`目录,`benchmark_core`目录,`game_env.bench.js`和`bench_entrance.ts`是前端测试所必须的文件,详见`pi_utils/benchs`
- 测试文件以`.bench.ts`结尾

# 运行
- scripts/install.bat
- scripts/build.bat
- scripts/start_server.bat
- 前端运行 http://127.0.0.1/boot/bench.html 控制台看打印结果
- 后端运行 TODO


### 示例

**benchs/math/math.bench.ts**

```
import * as Benchmark from "benchmark/benchmark";
import { completeLog } from "../bench_entrance";
import { cycleLog } from "../bench_entrance";
import { round } from "math/math";


const suite = new Benchmark.Suite("Benchmark Test example: ");
suite
    .add('math/math.round', function () {
        round(1.3456, 2);
    })
    .on('cycle', function (event) {
        cycleLog(event.currentTarget, event.target);
    }).on('complete', function () {
        completeLog(this)
    })
    .run({ 'async': true });
```



### 更多示例

```

import * as Benchmark from "benchmark/benchmark";


function cycleLog(suite, bench) {
    // console.log(`${suite.name}`, String(bench));
    let period = bench.times.period;  // s
    let count = 3;
    while (count) {
        if (period >= 1) break;
        period *= 1000;
        count--;
    }
    const timeUnits = ["ns", "us", "ms", "s"]
    console.log(`${suite.name} ${bench.name} ${period.toFixed(3)}${timeUnits[count]} once excute ±${bench.stats.rme.toFixed(2)}% （${bench.stats.sample.length} runs sampled）`);
}

function completeLog(suite) {
    console.log(`${suite.name} completed!!`);
}

/**
 * 测试同步函数
 */
function syncFunc() {
    return 'Hello World!'.indexOf('o') > -1;
}


/**
 * 测试异步函数
 */
function asyncFunc() {
    return new Promise(resolve => {
        setTimeout(resolve, 20)
    });
}

/**
 * 测试传参函数
 * @param {*} obj 
 */
function funcParams(obj) {
    return `${obj.name} ${obj.age}`
}



const suite = new Benchmark.Suite("Benchmark Test example: ");
suite
    .add('SyncFunc', syncFunc)
    .add('AsyncFunc', function (deferred) {
        asyncFunc().then(() => {
            deferred.resolve();
        })
    }, { defer: true })
    .add('funcParams', (function () {
        const zhangsan = { name: "zhangsan", age: 30 };
        return function () {
            funcParams(zhangsan)
        };
    })())
    .on('cycle', function (event) {
        cycleLog(event.currentTarget, event.target);
    }).on('complete', function () {
        completeLog(this)
    })
    .run({ 'async': true });
```