import { ResRef } from "./res_mgr/mod";

/**
 * JS 层资源引用对象
 */
export class Res<A, D> {
    /**
     * 底层引用
     * @hidden
     */
    private context: ResRef;
    /**
     * 资源名称
     * @hidden
     */
    private _name: string;
    /**
     * 资源类型
     * @hidden
     */
    private _type: string;
    /**
     * 资源参数
     */
    public args: A;
    /**
     * 资源名称/路径
     */
    public get name() {
        return this._name;
    }
    /**
     * 资源类型
     */
    public get type() {
        return this._type;
    }
    /**
     * 资源数据
     */
    public get link(): D {
        if (this.context) {
            return this.context.link().data;
        }
        return undefined;
    }
    /**
     * 修改资源数据
     * @hidden
     */
    public set link(value: D) {
        if (this.context) {
             this.context.link().data = value;
        }
    }
    /**
     * 资源 - 键
     * @hidden
     */
    private _key: string = '';
    /**
     * 资源 - 键
     */
    public get key() {
        if (!this._key) {
            this._key = genKey(this._type, this._name);
        }
        return this._key;
    }
    /**
     * 创建资源引用
     * @param name 资源路径
     * @param type 资源类型
     * @param link 资源底层引用
     * @param args 资源参数
     */
    public create(name: string, type: string, link: ResRef, args: any) {
        this._name = name;
        this._type = type;
        this.context = link;
        this.args = args;
        this._key = genKey(this._type, this._name);
    }
    /**
     * 释放资源引用
     */
    public free() {
        if (this.context)  {
            this.context.free();
            this.context = undefined;
        } else {
            
        }
    }
    /**
     * 获取资源底层引用
     */
    public getContext() {
        return this.context;
    }
}

/**
 * 创建资源键名
 * @param type 资源类型
 * @param name 资源名称
 */
export function genKey(type: string, name: string) {
    return `${type}:${name}`;
};