declare var module: any;
let wasm;

const heap = new Array(32).fill(undefined);

heap.push(undefined, null, true, false);

function getObject(idx) { return heap[idx]; }

let heap_next = heap.length;

function dropObject(idx) {
    if (idx < 36) return;
    heap[idx] = heap_next;
    heap_next = idx;
}

function takeObject(idx) {
    const ret = getObject(idx);
    dropObject(idx);
    return ret;
}

function addHeapObject(obj) {
    if (heap_next === heap.length) heap.push(heap.length + 1);
    const idx = heap_next;
    heap_next = heap[idx];

    heap[idx] = obj;
    return idx;
}

let cachedTextDecoder = new TextDecoder('utf-8', { ignoreBOM: true, fatal: true });

cachedTextDecoder.decode();

let cachegetUint8Memory0 = null;
function getUint8Memory0() {
    if (cachegetUint8Memory0 === null || cachegetUint8Memory0.buffer !== wasm.memory.buffer) {
        cachegetUint8Memory0 = new Uint8Array(wasm.memory.buffer);
    }
    return cachegetUint8Memory0;
}

function getStringFromWasm0(ptr, len) {
    return cachedTextDecoder.decode(getUint8Memory0().subarray(ptr, ptr + len));
}

/* tslint:disable */
/* eslint-disable */

export interface Res {
	destroy():void;
	[index: string]: any;
}


/**
* 资源管理器
*/
export class ResMgr {
	ptr: number;

    static __wrap(ptr) {
        const obj = Object.create(ResMgr.prototype);
        obj.ptr = ptr;

        return obj;
    }

    free() {
        const ptr = this.ptr;
        this.ptr = 0;

        wasm.__wbg_resmgr_free(ptr);
    }
    /**
    * 创建资源管理器的实例
    * @param {number} total_capacity
    */
    constructor(total_capacity) {
        var ret = wasm.resmgr_new(total_capacity);
        return ResMgr.__wrap(ret);
    }
    /**
    * 创建一个资源， 如果资源已经存在，程序会崩溃
    * @param {number} ty
    * @param {number} min_capacity
    * @param {number} max_capacity
    * @param {number} time_out
    */
    register_res(ty, min_capacity, max_capacity, time_out) {
        wasm.resmgr_register_res(this.ptr, ty, min_capacity, max_capacity, time_out);
    }
    /**
    * 创建一个资源， 如果资源已经存在，程序会崩溃
    * @param {number} ty
    * @param {number} key
    * @param {Res} res
    * @param {number} cost
    * @returns {ResRef}
    */
    create_res(ty, key, res, cost) {
        var ret = wasm.resmgr_create_res(this.ptr, ty, key, addHeapObject(res), cost);
        return ResRef.__wrap(ret);
    }
    /**
    * 获取资源
    * @param {number} ty
    * @param {number} key
    * @returns {ResRef | undefined}
    */
    get_res(ty, key) {
        var ret = wasm.resmgr_get_res(this.ptr, ty, key);
        return ret === 0 ? undefined : ResRef.__wrap(ret);
    }
    /**
    * 整理方法， 将无人使用的资源放入到LruCache， 清理过时的资源
    * 就是LruMgr有总内存上限， 按权重分给其下的LRU。 如果有LRU有空闲， 则会减少其max_size, 按权重提高那些满的LRU的max_size
    * @param {number} now
    */
    collect(now) {
        wasm.resmgr_collect(this.ptr, now);
    }
}
/**
* 资源
*/
export class ResRef {
	ptr: number

    static __wrap(ptr) {
        const obj = Object.create(ResRef.prototype);
        obj.ptr = ptr;

        return obj;
    }

    free() {
        const ptr = this.ptr;
        this.ptr = 0;

        wasm.__wbg_resref_free(ptr);
    }
    /**
    * @returns {any}
    */
    link() {
        var ret = wasm.resref_link(this.ptr);
        return takeObject(ret);
    }
}

async function load(module, imports) {
    if (typeof Response === 'function' && module instanceof Response) {

        if (typeof WebAssembly.instantiateStreaming === 'function') {
            try {
                return await WebAssembly.instantiateStreaming(module, imports);

            } catch (e) {
                if (module.headers.get('Content-Type') != 'application/wasm') {
                    console.warn("`WebAssembly.instantiateStreaming` failed because your server does not serve wasm with `application/wasm` MIME type. Falling back to `WebAssembly.instantiate` which is slower. Original error:\n", e);

                } else {
                    throw e;
                }
            }
        }

        const bytes = await module.arrayBuffer();
        return await WebAssembly.instantiate(bytes, imports);

    } else {

        const instance = await WebAssembly.instantiate(module, imports);

        if (instance instanceof WebAssembly.Instance) {
            return { instance, module };

        } else {
            return instance;
        }
    }
}

async function init(input) {
    // if (typeof input === 'undefined') {
    //     input = import.meta.url.replace(/\.js$/, '_bg.wasm');
    // }
    const imports: any = {};
    imports.wbg = {};
    imports.wbg.__wbg_destroy_246b7cd9634fe431 = function(arg0) {
        getObject(arg0).destroy();
    };
    imports.wbg.__wbindgen_object_drop_ref = function(arg0) {
        takeObject(arg0);
    };
    imports.wbg.__wbindgen_object_clone_ref = function(arg0) {
        var ret = getObject(arg0);
        return addHeapObject(ret);
    };
    imports.wbg.__wbindgen_throw = function(arg0, arg1) {
        throw new Error(getStringFromWasm0(arg0, arg1));
    };

    if (typeof input === 'string' || (typeof Request === 'function' && input instanceof Request) || (typeof URL === 'function' && input instanceof URL)) {
        input = fetch(input as string);
    }

    const r = await load(await input, imports);

    wasm = r.instance.exports;
	(<any>init).__wbindgen_wasm_module = r.module;
	
	if((<any>module).postRun) {
		module.postRun();
	}

    return wasm;
}

Promise.resolve().then(() => {
	init(module.wasmModule);
})

