/*
 * 小堆
 * 支持删除和更新
 */

// ============================== 导出

/**
 * 小堆
 */
export class Heap<T> {

	private array: T[] = [];                // 堆的实际数组，用来表示完全二叉树
	private cmp: (a: T, b: T) => number;    // 比较函数

	constructor(cmp: (a: T, b: T) => number) {
		this.cmp = cmp;
	}

	/**
	 * 判空
	 */
	public empty() {
		return this.array.length === 0;
	}

	/**
	 * 返回内部数组
	 */
	public getImpl() {
		return this.array;
	}

	/**
	 * 插入元素
	 */
	public insert(value: T) {
		this.array.push(value);
		this.up(this.array.length - 1);
	}

	/**
	 * 查找指定元素的位置
	 */
	public findIndex(predicate: (value: T, index: number, obj: T[]) => unknown, thisArg?: any) {
		return this.array.findIndex(predicate, thisArg);
	}
	/**
	 * 删除元素
	 */
	public remove(value: T) {
		let index = this.array.indexOf(value);
		index >= 0 && this.removeByIndex(index);
	}
	/**
	 * 删除指定位置的元素
	 */
	public removeByIndex(index: number) {
		// 把最后的叶子赋值给index位置
		const len = this.array.length - 1;
		if (index > len) return;
		const r = this.array[index];
		if(len > 0) {
			this.array[index] = this.array[len];
			this.down(index, len);
		}
		this.array.length = len;
		return r
	}
	/**
	 * 用过滤函数清除不要的元素， 过滤函数返回true表示保留
	 */
	public filter(f: (e: T) => boolean) {
		for(let i = this.array.length - 1; i >= 0; i--) {
			if(!f(this.array[i])) {
				this.removeByIndex(i);
			};
		}
	}

	/**
	 * 获得但不弹出堆顶元素
	 */
	public get() {
		return this.array[0];
	}

	/**
	 * 删除堆顶元素并返回
	 */
	public pop() {
		return this.removeByIndex(0)
	}

	/**
	 * 清空
	 */
	public clear() {
		this.array.length = 0;
	}

	/**
	 * 下沉
	 */
	private down(curr: number, len: number) {
		const arr = this.array;
		const element = arr[curr];
		let left = curr * 2 + 1;
		let right = left + 1;
		while (left < len) {
			// 选择左右孩子的最小值作为比较
			let child = left;
			let e = arr[left];
			if (right < len && this.cmp(arr[right], e) < 0) {
				child = right;
				e = arr[right];
			}

			// 待选择的值比孩子大，则将孩子移到当前的槽
			if (this.cmp(element, e) <= 0) {
				break;
			} else {
				arr[curr] = e;

				// 往下迭代
				curr = child;
				left = curr * 2 + 1;
				right = left + 1;
			}
		}
		arr[curr] = element;
	}

	/**
	 * 上朔
	 */
	private up(curr: number) {
		const arr = this.array;
		const element = arr[curr];
		let parent = Math.floor((curr - 1) / 2);
		while (parent >= 0) {
			let e = arr[parent];
			if(this.cmp(element, e) >= 0) {
				break
			}
			arr[curr] = e;
			// 往上迭代
			curr = parent;
			parent = Math.floor((curr - 1) / 2);
		}
		arr[curr] = element;
	}
}