# 单元测试

# libs
- [jasmine](https://github.com/jasmine/jasmine)


[jasmine 文档](https://jasmine.github.io/)

## 约定
- 目录名为tests,与src目录同级
    + tests目录下`boot`目录,`jasmine_core`目录,`game_env.test.js`和`test_entrance.ts`是前端测试所必须的文件,详见`pi_utils/tests`
- 测试文件以`.test.ts`结尾

# 运行
- scripts/install.bat
- scripts/build.bat
- scripts/start_server.bat
- 前端运行 http://127.0.0.1/boot/test.html 控制台看打印结果
- 后端运行 TODO


### 示例

**benchs/math/math.bench.ts**

```
import { round } from "math/math";

describe("Jasmine UnitTest math/math.ts ==>", function () {
    it("round :", function () {
        expect(round(1.3456, 2)).toBe(1.35);
    });
});
```



### 更多示例

```


/**
 * 测试同步函数
 */
function syncFunc() {
    return 'Hello World!'.indexOf('o') > -1;
}


/**
* 测试异步函数
*/
function asyncFunc() {
    return new Promise(resolve => {
        setTimeout(() => { resolve("Hello Jasmine") }, 20)
    });
}

/**
 * 测试异步callback形式的函数
 */
function asyncCallBackFunc(callback) {
    setTimeout(() => { callback("Hello Jasmine") }, 20)
}

/**
* 测试传参函数
* @param {*} obj 
*/
function funcParams(obj) {
    return `${obj.name} ${obj.age}`
}


/**
 * 测试同步函数
 */
describe("Jasmine UnitTest Sync example ==>", function () {
    it("equal test:", function () {
        expect(syncFunc()).toBe(true);
    });
    it("not equal test:", function () {
        expect(syncFunc()).not.toBe(false);
    });
});


/**
 * 测试异步
 * 1. async/await 
 * 2. promise
 * 3. callbacks
 */
describe("Jasmine UnitTest Async example ==>", function () {
    // TODO pi_serv parse eroor
    // it("async/await:", async function () {
    //     const result = await asyncFunc();
    //     expect(result).toBe("Hello Jasmine");
    // });
    it("return promises:", function () {
        return asyncFunc().then(result => {
            expect(result).toBe("Hello Jasmine");
        })
    });

    it("use callbacks:", function (done) {
        asyncCallBackFunc(function (result) {
            expect(result).toBe("Hello Jasmine");
            done();
        })
    });
});


/**
 * params transfer
 * this 关键字在每一轮的beforeEach/it/afterEach中是同一个对象
 */
describe("Jasmine UnitTest transfer params example ==>", function () {
    let zhangsan = { name: "zhangsan", age: 30 };
    // todo 不知道为啥 在pi_serv上运行beforeAll会在测试完成后出错
    // beforeAll(function () {  // once
    //     console.log("beforeAll");
    //     this.foo = 1;
    // });
    beforeEach(function () {   // every it excute
        console.log("beforeEach");
        this.foo += 1;
        this.zhangsan = zhangsan;
    });
    afterEach(function () {  // every it excute 
        console.log("afterEach");
    });
    // afterAll(function () {   // once
    //     console.log("afterAll");
    //     zhangsan = null;
    // });

    it("local variable:", function () {
        expect(this.foo).toBe(2);
        expect(funcParams(zhangsan)).toBe("zhangsan 30");
    });
    it("the this keyword:", function () {
        expect(this.foo).toBe(2);
        expect(funcParams(this.zhangsan)).toBe("zhangsan 30");
    });

});
```